<?php

namespace Drupal\views_vcards\Plugin\views\row;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\views\Plugin\views\row\RowPluginBase;
use Drupal\views_vcards\PropertyPluginManager;
use Drupal\views_vcards\VcardPropertyPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Renders a single vCard from selected fields.
 *
 * @ingroup views_row_plugins
 *
 * @ViewsRow(
 *   id = "views_vcard_fields",
 *   title = @Translation("vCard"),
 *   help = @Translation("Combines fields into a vCard."),
 *   theme = "views_vcards_view_row_vcard",
 *   display_types = {"views_vcard"}
 * )
 */
class ViewsVcardsRowPluginVcard extends RowPluginBase implements TrustedCallbackInterface {

  /**
   * Does the row plugin support to add fields to it's output.
   *
   * @var bool
   */
  protected $usesFields = TRUE;

  /**
   * The propertyPluginManager.
   *
   * @var \Drupal\views_vcards\PropertyPluginManager
   */
  protected $propertyPluginManager;

  /**
   * An array of instantiated plugins.
   *
   * @var \Drupal\views_vcards\VcardPropertyPluginInterface[]
   */
  protected $plugins;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PropertyPluginManager $plugin_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->propertyPluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.views_vcards.property')
    );
  }

  /**
   * Instantiates all VcardPropertyPlugins.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function initPlugins() {
    $plugin_configuration = $this->options['vcard_properties'];

    foreach ($plugin_configuration as $plugin_instance) {
      if (!isset($plugin_instance['plugin'])) {
        continue;
      }
      $property_plugin_type = $plugin_instance['plugin'];
      $property_plugin_configuration = ($plugin_instance['configuration'] ?? []);

      /** @var \Drupal\views_vcards\VcardPropertyPluginInterface $plugin */
      $plugin = $this->propertyPluginManager->createInstance($property_plugin_type, $property_plugin_configuration);
      $plugin->setView($this->view);

      $this->plugins[] = $plugin;
    }
  }

  /**
   * Gets an array of instantiated plugins.
   *
   * @return \Drupal\views_vcards\VcardPropertyPluginInterface[]
   *   Array of initialized vCard property plugins.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getPlugins() {
    if (empty($this->plugins)) {
      $this->initPlugins();
    }
    return $this->plugins;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['#tree'] = TRUE;
    $form['#prefix'] = '<div id="views-vcards-assignment-form-ajax-wrapper">';
    $form['#suffix'] = '</div>';

    $form['vcard_properties'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Property'),
        $this->t('Settings'),
      ],
    ];

    if (!empty($form_state->getUserInput()['row_options']['actions']['add']['property'])) {
      $this->options['vcard_properties'][] = [
        'plugin' => $form_state->getUserInput()['row_options']['actions']['add']['property'],
      ];

      $this->initPlugins();
    }

    // Populate all the plugin settings forms.
    foreach ($this->getPlugins() as $delta => $plugin) {
      $form['vcard_properties'][$delta] = [
        'property' => [
          '#markup' => $plugin->getPluginDefinition()['property'],
        ],
        'configuration' => [
          '#type' => 'container',
        ],
        'plugin' => [
          '#type' => 'value',
          '#value' => $plugin->getPluginId(),
        ],
      ];

      $subform_state = SubformState::createForSubform($form['vcard_properties'][$delta]['configuration'], $form, $form_state);
      $form['vcard_properties'][$delta]['configuration'] = $plugin->buildConfigurationForm($form['vcard_properties'][$delta]['configuration'], $subform_state);
    }

    // Create the new property drop down.
    $available_plugin_definitions = $this->propertyPluginManager->getDefinitions();
    foreach ($available_plugin_definitions as &$def) {
      $def = $def['property'] . " (" . $def['title'] . ")";
    }
    $form['actions']['add'] = [
      'property' => [
        '#type' => 'select',
        '#options' => $available_plugin_definitions,
        '#empty_option' => $this->t('- Add property -'),
        '#empty_value' => '',
        '#default_value' => NULL,
      ],
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Add new property'),
        '#id' => 'vcards-add-property',
        '#ajax' => [
          'url' => NULL,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    foreach ($this->getPlugins() as $delta => $plugin) {
      if ($plugin instanceof VcardPropertyPluginInterface) {
        $subform_state = SubformState::createForSubform($form['vcard_properties'][$delta]['configuration'], $form, $form_state);
        $plugin->validateConfigurationForm($form['vcard_properties'][$delta]['configuration'], $subform_state);
      }
    }
    parent::validateOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();

    // Check if we were adding a new vcard property.
    if (!empty($triggering_element['#id']) && $triggering_element['#id'] == 'vcards-add-property') {
      $view = $form_state->get('view');
      // Return to this form.
      $view->addFormToStack($form_state->get('form_key'), $form_state->get('display_id'), 'row_options');
    }
    else {
      foreach ($this->getPlugins() as $delta => $plugin) {
        if ($plugin instanceof VcardPropertyPluginInterface) {
          $subform_state = SubformState::createForSubform($form['vcard_properties'][$delta]['configuration'], $form, $form_state);
          $plugin->submitConfigurationForm($form['vcard_properties'][$delta]['configuration'], $subform_state);
        }
      }

      // Unset the 'add' line form the options.
      $form_state->unsetValue(['row_options', 'actions']);

      parent::submitOptionsForm($form, $form_state);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render($row) {
    static $row_index;
    if (!isset($row_index)) {
      $row_index = 0;
    }

    $properties = [];
    $full_name = '';

    /** @var \Drupal\views_vcards\VcardPropertyPluginBase $plugin */
    foreach ($this->getPlugins() as $plugin) {
      if ($content = $plugin->render($row_index)) {
        $properties[] = $content;

        // Get the full name from the FN property.
        if ($plugin->getPluginId() == 'fn') {
          $full_name = $plugin->getFieldValue($row_index, $plugin->getConfiguration('views_field'));
        }
      }
    }
    $row_index++;

    $row = implode("\n", $properties);

    return [
      '#theme' => $this->themeFunctions(),
      '#view' => $this->view,
      '#options' => $this->options,
      '#row' => $row,
      '#vcard_fullname' => $full_name,
      '#field_alias' => $this->field_alias ?? '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    // @todo Determine other dependencies here.
    return $dependencies;
  }

}
