<?php

namespace Drupal\views_vcards;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\views\ViewExecutable;

/**
 * Base class for all vCard property plugins.
 */
abstract class VcardPropertyPluginBase extends PluginBase implements VcardPropertyPluginInterface {

  /**
   * The view.
   *
   * @var \Drupal\views\ViewExecutable
   */
  protected $view;

  /**
   * {@inheritdoc}
   */
  public function setView(ViewExecutable $view) {
    $this->view = $view;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration($key = NULL) {
    // Return the value for a key if it exists.
    if (isset($this->configuration[$key])) {
      return $this->configuration[$key];
    }
    // Return the collection on NULL
    elseif ($key === NULL) {
      return $this->configuration;
    }
    // Otherwise return nothing.
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = NestedArray::mergeDeepArray([$this->defaultConfiguration(), $configuration], TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'plugin' => '',
      'home_work_type' => '',
      'views_field' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // Init the style plugin so that we retrieve the available fields.
    $this->view->initStyle();

    $form['views_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Views field'),
      '#description' => $this->t('The views field containing the value for this property.'),
      '#options' => $this->view->rowPlugin->displayHandler->getFieldLabels(),
      '#empty_option' => $this->t('- Select -'),
      '#default_value' => $this->getConfiguration('views_field'),
      '#required' => TRUE,
    ];

    if (!empty($this->getPluginDefinition()['supportsHomeWorkType'])) {
      $form['home_work_type'] = [
        '#type' => 'select',
        '#title' => $this->t('Specification'),
        '#description' => $this->t('Indicates if this property specifically applies to a home or workplace.'),
        '#options' => [
          'home' => $this->t('Home'),
          'work' => $this->t('Work'),
        ],
        '#empty_option' => $this->t('Generic'),
        '#empty_value' => '',
        '#default_value' => $this->getConfiguration('home_work_type'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // @todo This appears to be doing nothing. Check if it is still needed.
    $configuration = $form_state->cleanValues()->getValues();
    $this->setConfiguration($configuration);
  }

  /**
   * Returns the property's content string.
   *
   * @param int $index
   *   The index (row number) of the field.
   * @param string[] $params
   *   Optional array of parameters to add to this property. (e.g., 'TYPE=foo')
   * @param string $group
   *   Optional group.
   *
   * @return string|null
   *   The property's content line. NULL if the property is empty.
   */
  public function prepareOutputString($index, $params = [], $group = '') {
    $output = NULL;
    $types = [];

    $value = $this->getFieldValue($index, $this->getConfiguration('views_field'));
    if (!empty($value)) {
      if (!empty($this->getConfiguration('home_work_type'))) {
        $types[] = $this->getConfiguration('home_work_type');
      }

      // Remove possible null values that can come from home_work_type.
      $types = array_values(array_filter($types));

      if ($types) {
        $params[] = 'TYPE=' . implode(',', $types);
      }

      // Build the vCard content line according the ABNF Format Definition.
      $property = $this->getPluginDefinition()['property'];
      $output = ($group ? $group . '.' : '') .
        $property .
        (!empty($params) ? ';' . implode(';', $params) : '') .
        ':' . $value;
    }

    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function render($index, $params = [], $group = '') {
    $output = NULL;

    if ($output_string = $this->prepareOutputString($index, $params, $group)) {
      // Finally return a line wrapped at 75 characters.
      $output = substr($output_string, 0, 75) . "\r\n  ";
      $output .= chunk_split(substr($output_string, 75), 73, "\r\n  ");
      // Remove last newline.
      $output = rtrim($output);
    }

    return $output;
  }

  /**
   * Retrieves a views field value from the style plugin.
   *
   * @param int $index
   *   The index count of the row as expected by views_plugin_style::getField().
   * @param string $field_id
   *   The ID assigned to the required field in the display.
   *
   * @return string|null|\Drupal\Component\Render\MarkupInterface
   *   An empty string if there is no style plugin, or the field ID is empty.
   *   NULL if the field value is empty. If neither of these conditions apply,
   *   a MarkupInterface object containing the rendered field value.
   */
  public function getFieldValue($index, $field_id) {
    if (empty($this->view->style_plugin) || !is_object($this->view->style_plugin) || empty($field_id)) {
      return '';
    }
    return $this->view->style_plugin->getField($index, $field_id);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    // @todo Implement calculateDependencies() method.
  }

}
